library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

library work;
use work.seg7mux.all;
use work.clockgen.all;
use work.btn_debounce.all;

entity ram is
    Port ( clk50m 	: in	 	std_logic;
           res 			: in	 	std_logic;

					 swin  		: in	 	std_logic_vector ( 7 downto 0);
					 pbtn   	: in	 	std_logic_vector ( 2 downto 0);

					 ledout		: out 	std_logic_vector ( 7 downto 0);
					 digout		: out		std_logic_vector ( 3 downto 0);
					 segout		: out		std_logic_vector ( 7 downto 0);

					 raddr		: out 	std_logic_vector (17 downto 0);
					 rdata 		: inout std_logic_vector ( 7 downto 0);
					 ce				: out		std_logic;
					 we				: out		std_logic;	
					 oe				: out		std_logic;
					 lb				: out		std_logic;
					 ub				: out		std_logic
				 );
end ram;

architecture Behavioral of ram is

-- komponenten

	-- stellt aus 50mhz clock folgende clocks zur verfgung : 1mhz, 10khz, 1 khz, 100 hz, 1 hz
	component clockgen is
	   port (
					 	clk50m  	: in  std_logic;
					 	res				: in 	std_logic;
					 	clk1mo		: out std_logic;
					 	clk10ko		: out std_logic;
					 	clk1ko		: out std_logic;
					 	clk100o		: out std_logic;
					 	clk1o			: out std_logic
					);
	end component;

	-- demultiplext einen 16 bit wert auf 4 7segment anzeigen + getrennter punkt-behandlung
	component seg7mux is
	  port (
						clk1k   	: in 	std_logic;
						res				:	in	std_logic;
						value16		: in 	std_logic_vector (15 downto 0);
						dots4			: in  std_logic_vector ( 3 downto 0);
						segout  	: out	std_logic_vector ( 7 downto 0);
						digout  	: out	std_logic_vector ( 3 downto 0)
					);
	end component;

	-- taster entprellen (100 hz signal !!!)
	component btn_debounce is
   	 port ( clk100 		: in std_logic;
    	      res 			: in std_logic;
			 		  hold			: in std_logic;
            i 				: in std_logic;
            o 				: out std_logic
					);
	end component;

-- interne signale

	-- clocks
	signal clock1k  	: std_logic;
	signal clock1m  	: std_logic;
	signal clock100 	: std_logic;

	-- "ui" display & btns & dots (input)
	signal data16			: std_logic_vector (15 downto 0);	
	signal dots4			: std_logic_vector ( 3 downto 0);
	signal btns 			: std_logic_vector ( 2 downto 0);

	-- "ui" addr & dtaen bus, daten anzeige & modus
	signal addr				: std_logic_vector ( 7 downto 0);	
	signal data_sw  	: std_logic_vector ( 7 downto 0);
	signal data_led 	: std_logic_vector ( 7 downto 0);

	signal mode				: std_logic_vector ( 2 downto 0);

	-- "hw"	ram statemachine
	signal state			: std_logic_vector ( 2 downto 0);  -- statemachine fr lesen und schreiben
	signal data_in  	: std_logic_vector ( 7 downto 0);

begin

	-- clocks generieren

  clocks : clockgen port map (
										  clk50m 	  => clk50m,
											res			 	=> res,
											clk1mo		=> clock1m,
											clk1ko		=> clock1k,
											clk100o	  => clock100
										);

  -- 3 taster entprellen (der 4. ist reset)

  btn1 : btn_debounce port map (
											clk100		=> clock100,
											res				=> res,
											hold			=> '0',					-- nur flanke
											i					=> pbtn (0),
											o					=> btns (0)
										);

  btn2 : btn_debounce port map (
											clk100		=> clock100,
											res				=> res,
											hold			=> '0',
											i					=> pbtn (1),
											o					=> btns (1)
										);

  btn3 : btn_debounce port map (
											clk100		=> clock100,
											res				=> res,
											hold			=> '0',
											i					=> pbtn (2),
											o					=> btns (2)
										);


	-- eigentlicher ui prozess

	process (clk50m, res)
	begin
	  if res = '1' then
		  -- alles reset
			data16 		<= (others => '0');
  		dots4			<= (others => '0');
			data_in   <= (others => '0');
			data_sw 	<= (others => '0');
			data_led	<= (others => '0');
			
			ledout 		<= (others => '1');

			mode			<= (others => '0');

			rdata 		<= (others => 'Z');

			state			<= (others => '0');
			we				<= '1';
			ce				<= '1';
			oe				<= '1';
			lb				<= '0';
			ub				<= '0';

		elsif clk50m'event and clk50m = '1' then

			-- ram statemachine handeln

			if clock1m = '1' then
				case state is
					--- idle state
				  when "000" 	=> rdata 		<= (others => 'Z');

					--- write state

					when "001"  => 	raddr (7 downto 0) <= addr; 
													rdata 	<= data_sw; 
													ce 			<= '0'; 
													oe 			<= '1'; 
													we 			<= '1'; 
													lb			<= '1';
													ub			<= '0';
													state 	<= "010";

					when "010"	=> 	ce 			<= '1'; 
													we 			<= '0'; 
													state 	<= "011";

					when "011"	=> 	ce 			<= '0'; 
													rdata 	<= data_sw; 
													state 	<= "000";

					--- read state

					when "100"  => 	raddr (7 downto 0) <= addr; 
													rdata 	<= (others => 'Z'); 
													ce 			<= '0'; 
													oe 			<= '0'; 
													we 			<= '1';													 
													lb			<= '1';
													ub			<= '0';
													state 	<= "101";

					when "101"	=> 	ce 			<= '1'; 
													we 			<= '1';
													state 	<= "110";

					when "110"	=> 	ce 			<= '0'; 
													oe 			<= '1';
													data_in <= rdata;
													state 	<= "000";
	
					when others => state 		<= "000";
				end case;
			end if;

			-- taster & ui verarbeiten

			if clock100 = '1' then

				-- mode erhhen (also funktion)
				if btns (1) = '1' then
					mode <= mode + '1';
					if mode = "011" then
						mode <= "000";
					end if;
				end if;

				if btns (2) = '1' then
					mode <= mode + '1';
					if mode = "011" then
						mode <= "000";
					end if;
				end if;

				-- gewhlte fkt ausfhren
				if btns (0) = '1' then
					case mode is
						when "000"  => addr		 	<= swin;	-- addresse bernehmen
						when "001"  => data_sw 	<= swin;  -- daten bernehmen
						when "010" 	=> state 		<= "001";	-- write state antriggern
						when "011" 	=> state 		<= "100";	-- read  state antriggern
						when others	=> null;
					end case;
				end if;


				-- ab hier leds & 7seg

				-- leds muxen	(funktionsanzeige, gibt nur 4 stck)
				case mode is
				  when "000" 	=> ledout <= "00000001";
				  when "001" 	=> ledout <= "00000010";
				  when "010" 	=> ledout <= "00000100";
				  when "011" 	=> ledout <= "00001000";
				  when others	=> ledout <= "00000000"; mode <= "000";	 -- falls ungltiger mode -> mode reset
				end case;

				-- data fr 7seg muxen (read-input oder addresse)
				case mode is
					when "011" 	=> data_led <= data_in;		-- nur bei read auch den inhalt anzeigen
					when others => data_led <= data_sw;		-- sonst zuletzt eingegebenes datenwort
				end case;

				-- dots setzen (address oder daten auswahl ?)
				case mode is
					when "000" 	=> dots4    <= "1100";
					when "001" 	=> dots4    <= "0011";
					when others => dots4	  <= "0000";
				end case;

			end if;

			-- 7seg display aufbereiten
			data16 	<= addr & data_led;

			-- rest vom addr-bus auffllen
			raddr (17 downto 8) <= (others => '0');

		end if;
	end process;

	-- aktuelles "display" (data16 + dots4) darstellen

	seg7muxer : seg7mux port map (
											clk1k 	=> clock1k,
											res			=> res,
											value16	=> data16,
											dots4		=> dots4,
											segout	=> segout,
											digout	=> digout
										);
end Behavioral;
